@@ -1069,7 +1069,7 @@ xfce_panel_plugin_menu_get (XfcePanelPlugin *plugin)
 
       /* create a panel submenu item */
       submenu = gtk_menu_new ();
-      item = gtk_menu_item_new_with_mnemonic (_("_Xfce Panel"));
+      item = gtk_menu_item_new_with_mnemonic (_("Pane_l"));
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
       gtk_widget_show (item);
@@ -1505,8 +1505,9 @@ xfce_panel_plugin_get_unique_id (XfcePanelPlugin *plugin)
  *
  * Returns: the property base for the xfconf channel userd by a plugin.
  *
- * See also: xfconf_channel_new_with_property_base() and
- *           XFCE_PANEL_PLUGIN_CHANNEL_NAME.
+ * See also: xfconf_channel_new_with_property_base.
+ *           XFCE_PANEL_PLUGIN_CHANNEL_NAME and
+ *           xfce_panel_get_channel_name
  **/
 G_CONST_RETURN gchar *
 xfce_panel_plugin_get_property_base (XfcePanelPlugin *plugin)
