@@ -60,18 +60,19 @@ static gboolean   opt_quit = FALSE;
 static gboolean   opt_version = FALSE;
 static gchar     *opt_plugin_event = NULL;
 static gchar    **opt_arguments = NULL;
+static gboolean   sm_client_saved_state = FALSE;
 
 
 
-static gboolean callback_handler (const gchar  *name,
-                                  const gchar  *value,
-                                  gpointer      user_data,
-                                  GError      **error);
+static gboolean panel_callback_handler (const gchar  *name,
+                                        const gchar  *value,
+                                        gpointer      user_data,
+                                        GError      **error);
 
 
 
 /* command line options */
-#define PANEL_CALLBACK_OPTION G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, callback_handler
+#define PANEL_CALLBACK_OPTION G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, panel_callback_handler
 static GOptionEntry option_entries[] =
 {
   { "preferences", 'p', PANEL_CALLBACK_OPTION, N_("Show the 'Panel Preferences' dialog"), N_("PANEL-NUMBER") },
@@ -89,10 +90,10 @@ static GOptionEntry option_entries[] =
 
 
 static gboolean
-callback_handler (const gchar  *name,
-                  const gchar  *value,
-                  gpointer      user_data,
-                  GError      **error)
+panel_callback_handler (const gchar  *name,
+                        const gchar  *value,
+                        gpointer      user_data,
+                        GError      **error)
 {
   panel_return_val_if_fail (name != NULL, FALSE);
 
@@ -118,9 +119,45 @@ callback_handler (const gchar  *name,
 
 
 static void
-signal_handler_quit (gint signum)
+panel_signal_handler (gint signum)
 {
-  panel_dbus_service_exit_panel (FALSE);
+  panel_debug (PANEL_DEBUG_DOMAIN_MAIN,
+               "received signal %s <%d>, %s panel",
+               g_strsignal (signum), signum,
+               signum == SIGUSR1 ? "restarting" : "quiting");
+
+  panel_dbus_service_exit_panel (signum == SIGUSR1);
+}
+
+
+
+static void
+panel_sm_client_quit (XfceSMClient *sm_client)
+{
+  panel_return_if_fail (XFCE_IS_SM_CLIENT (sm_client));
+  panel_return_if_fail (!panel_dbus_service_get_restart ());
+
+  panel_debug (PANEL_DEBUG_DOMAIN_MAIN,
+               "terminate panel for session manager");
+
+  gtk_main_quit ();
+}
+
+
+
+static void
+panel_sm_client_save_state (XfceSMClient     *sm_client,
+                            PanelApplication *application)
+{
+  panel_return_if_fail (XFCE_IS_SM_CLIENT (sm_client));
+  panel_return_if_fail (PANEL_IS_APPLICATION (application));
+
+  panel_debug (PANEL_DEBUG_DOMAIN_MAIN,
+               "save configuration for session manager");
+
+  panel_application_save (application, TRUE);
+
+  sm_client_saved_state = TRUE;
 }
 
 
@@ -134,12 +171,17 @@ main (gint argc, gchar **argv)
   PanelDBusService *dbus_service;
   gboolean          succeed = FALSE;
   guint             i;
-  const gint        signums[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM };
+  const gint        signums[] = { SIGINT, SIGQUIT, SIGTERM, SIGABRT, SIGUSR1 };
   const gchar      *error_msg;
   XfceSMClient     *sm_client;
 
-  panel_debug (PANEL_DEBUG_DOMAIN_MAIN, "panel version %s with Xfce %s",
-               LIBXFCE4PANEL_VERSION, xfce_version_string ());
+  panel_debug (PANEL_DEBUG_DOMAIN_MAIN,
+               "version %s on gtk+ %d.%d.%d (%d.%d.%d), glib %d.%d.%d (%d.%d.%d)",
+               LIBXFCE4PANEL_VERSION,
+               gtk_major_version, gtk_minor_version, gtk_micro_version,
+               GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+               glib_major_version, glib_minor_version, glib_micro_version,
+               GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
 
   /* set translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
@@ -236,21 +278,26 @@ main (gint argc, gchar **argv)
   /* start session management */
   sm_client = xfce_sm_client_get ();
   xfce_sm_client_set_restart_style (sm_client, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
+  xfce_sm_client_set_priority (sm_client, XFCE_SM_CLIENT_PRIORITY_CORE);
   g_signal_connect (G_OBJECT (sm_client), "quit",
-      G_CALLBACK (signal_handler_quit), NULL);
+      G_CALLBACK (panel_sm_client_quit), NULL);
   if (!xfce_sm_client_connect (sm_client, &error))
     {
       g_printerr ("%s: Failed to connect to session manager: %s\n",
                   G_LOG_DOMAIN, error->message);
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
   /* setup signal handlers to properly quit the main loop */
   for (i = 0; i < G_N_ELEMENTS (signums); i++)
-    signal (signums[i], signal_handler_quit);
+    signal (signums[i], panel_signal_handler);
 
   application = panel_application_get ();
 
+  /* save the state before the quit signal if we can, this is a bit safer */
+  g_signal_connect (G_OBJECT (sm_client), "save-state",
+      G_CALLBACK (panel_sm_client_save_state), application);
+
   /* open dialog if we started from launch_panel */
   if (opt_preferences >= 0)
     panel_preferences_dialog_show (panel_application_get_nth_window (application, opt_preferences));
@@ -263,8 +310,9 @@ main (gint argc, gchar **argv)
   /* destroy all the opened dialogs */
   panel_application_destroy_dialogs (application);
 
-  /* save the configuration */
-  panel_application_save (application, TRUE);
+  /* only save if we do not quit by the sm client */
+  if (!sm_client_saved_state)
+    panel_application_save (application, TRUE);
 
   g_object_unref (G_OBJECT (application));
   g_object_unref (G_OBJECT (sm_client));
@@ -310,12 +358,12 @@ dbus_return:
               g_clear_error (&error);
 
               if (xfce_dialog_confirm (NULL, GTK_STOCK_EXECUTE, NULL,
-                                       _("Do you want to start the Xfce panel? If you do, make sure "
+                                       _("Do you want to start the panel? If you do, make sure "
                                          "you save the session on logout, so the panel is "
                                          "automatically started the next time you login."),
                                        _("No running instance of %s was found"), G_LOG_DOMAIN))
                 {
-                  panel_debug (PANEL_DEBUG_DOMAIN_MAIN, "start panel of asking user");
+                  panel_debug (PANEL_DEBUG_DOMAIN_MAIN, "user confirmed to start the panel");
                   goto launch_panel;
                 }
               else
