@@ -48,10 +48,14 @@ static void pager_buttons_screen_workspace_created   (WnckScreen    *screen,
 static void pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
                                                       WnckWorkspace *destroyed_workspace,
                                                       PagerButtons  *pager);
+static void pager_buttons_screen_viewports_changed   (WnckScreen    *screen,
+                                                      PagerButtons  *pager);
 static void pager_buttons_workspace_button_toggled   (GtkWidget     *button,
                                                       WnckWorkspace *workspace);
 static void pager_buttons_workspace_button_label     (WnckWorkspace *workspace,
                                                       GtkWidget     *label);
+static void pager_buttons_viewport_button_toggled    (GtkWidget     *button,
+                                                      PagerButtons  *pager);
 
 
 
@@ -82,6 +86,13 @@ enum
   PROP_ORIENTATION
 };
 
+enum
+{
+  VIEWPORT_X,
+  VIEWPORT_Y,
+  N_INFOS
+};
+
 
 
 XFCE_PANEL_DEFINE_TYPE (PagerButtons, pager_buttons, GTK_TYPE_TABLE)
@@ -187,6 +198,8 @@ pager_buttons_set_property (GObject      *object,
           G_CALLBACK (pager_buttons_screen_workspace_created), pager);
       g_signal_connect (G_OBJECT (pager->wnck_screen), "workspace-destroyed",
           G_CALLBACK (pager_buttons_screen_workspace_destroyed), pager);
+      g_signal_connect (G_OBJECT (pager->wnck_screen), "viewports-changed",
+          G_CALLBACK (pager_buttons_screen_viewports_changed), pager);
 
       pager_buttons_queue_rebuild (pager);
       break;
@@ -223,6 +236,8 @@ pager_buttons_finalize (GObject *object)
           G_CALLBACK (pager_buttons_screen_workspace_created), pager);
       g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
           G_CALLBACK (pager_buttons_screen_workspace_destroyed), pager);
+      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
+          G_CALLBACK (pager_buttons_screen_viewports_changed), pager);
 
       g_object_unref (G_OBJECT (pager->wnck_screen));
     }
@@ -234,6 +249,27 @@ pager_buttons_finalize (GObject *object)
 
 
 
+static gboolean
+pager_buttons_button_press_event (GtkWidget      *button,
+                                  GdkEventButton *event)
+{
+  guint modifiers;
+
+  panel_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (button), FALSE);
+
+  modifiers = event->state & gtk_accelerator_get_default_mod_mask ();
+
+  /* block toggle events on an active button */
+  if (event->button == 1
+      && modifiers != GDK_CONTROL_MASK
+      && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+    return TRUE;
+
+  return FALSE;
+}
+
+
+
 static gboolean
 pager_buttons_rebuild_idle (gpointer user_data)
 {
@@ -247,6 +283,13 @@ pager_buttons_rebuild_idle (gpointer user_data)
   WnckWorkspace *workspace;
   GtkWidget     *panel_plugin;
   GtkWidget     *label;
+  gint           workspace_width, workspace_height;
+  gint           screen_width, screen_height;
+  gint           viewport_x, viewport_y;
+  gboolean       viewport_mode = FALSE;
+  gint           n_viewports;
+  gint          *vp_info;
+  gchar          text[8];
 
   panel_return_val_if_fail (XFCE_IS_PAGER_BUTTONS (pager), FALSE);
   panel_return_val_if_fail (WNCK_IS_SCREEN (pager->wnck_screen), FALSE);
@@ -264,10 +307,43 @@ pager_buttons_rebuild_idle (gpointer user_data)
 
   n_workspaces = g_list_length (workspaces);
 
-  rows = CLAMP (1, pager->rows, n_workspaces);
-  cols = n_workspaces / rows;
-  if (cols * rows < n_workspaces)
-    cols++;
+  /* check if the user uses 1 workspace with viewports */
+  if (G_UNLIKELY (n_workspaces == 1
+      && wnck_workspace_is_virtual (WNCK_WORKSPACE (workspaces->data))))
+    {
+      workspace = WNCK_WORKSPACE (workspaces->data);
+
+      workspace_width = wnck_workspace_get_width (workspace);
+      workspace_height = wnck_workspace_get_height (workspace);
+      screen_width = wnck_screen_get_width (pager->wnck_screen);
+      screen_height = wnck_screen_get_height (pager->wnck_screen);
+
+      /* we only support viewports that are equally spread */
+      if ((workspace_width % screen_width) == 0
+          && (workspace_height % screen_height) == 0)
+        {
+          n_viewports = (workspace_width / screen_width) * (workspace_height / screen_height);
+
+          rows = CLAMP (1, pager->rows, n_viewports);
+          cols = n_workspaces / rows;
+          if (cols * rows < n_workspaces)
+            cols++;
+
+          viewport_mode = TRUE;
+        }
+      else
+        {
+          g_warning ("only viewports with equally distributed screens are supported: %dx%d & %dx%d",
+                     workspace_width, workspace_height, screen_width, screen_height);
+        }
+    }
+  else
+    {
+      rows = CLAMP (1, pager->rows, n_workspaces);
+      cols = n_workspaces / rows;
+      if (cols * rows < n_workspaces)
+        cols++;
+    }
 
   if (pager->orientation != GTK_ORIENTATION_HORIZONTAL)
     SWAP_INTEGER (rows, cols);
@@ -276,36 +352,84 @@ pager_buttons_rebuild_idle (gpointer user_data)
 
   panel_plugin = gtk_widget_get_ancestor (GTK_WIDGET (pager), XFCE_TYPE_PANEL_PLUGIN);
 
-  for (li = workspaces, n = 0; li != NULL; li = li->next, n++)
+  if (G_UNLIKELY (viewport_mode))
     {
-      workspace = WNCK_WORKSPACE (li->data);
+      panel_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
 
-      button = xfce_panel_create_toggle_button ();
-      if (workspace == active_ws)
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-      g_signal_connect (G_OBJECT (button), "toggled",
-          G_CALLBACK (pager_buttons_workspace_button_toggled), workspace);
-      xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (panel_plugin), button);
-      gtk_widget_show (button);
+      viewport_x = wnck_workspace_get_viewport_x (workspace);
+      viewport_y = wnck_workspace_get_viewport_y (workspace);
 
-      label = gtk_label_new (NULL);
-      g_signal_connect_object (G_OBJECT (workspace), "name-changed",
-          G_CALLBACK (pager_buttons_workspace_button_label), label, 0);
-      pager_buttons_workspace_button_label (workspace, label);
-      gtk_label_set_angle (GTK_LABEL (label),
-          pager->orientation == GTK_ORIENTATION_HORIZONTAL ? 0 : 270);
-      gtk_container_add (GTK_CONTAINER (button), label);
-      gtk_widget_show (label);
+      for (n = 0; n < n_viewports; n++)
+        {
+          vp_info = g_new0 (gint, N_INFOS);
+          vp_info[VIEWPORT_X] = (n % (workspace_height / screen_height)) * screen_width;
+          vp_info[VIEWPORT_Y] = (n / (workspace_height / screen_height)) * screen_height;
 
-      pager->buttons = g_slist_prepend (pager->buttons, button);
+          button = xfce_panel_create_toggle_button ();
+          if (viewport_x >= vp_info[VIEWPORT_X] && viewport_x < vp_info[VIEWPORT_X] + screen_width
+              && viewport_y >= vp_info[VIEWPORT_Y] && viewport_y < vp_info[VIEWPORT_Y] + screen_height)
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+          g_signal_connect (G_OBJECT (button), "toggled",
+              G_CALLBACK (pager_buttons_viewport_button_toggled), pager);
+          g_signal_connect (G_OBJECT (button), "button-press-event",
+              G_CALLBACK (pager_buttons_button_press_event), NULL);
+          xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (panel_plugin), button);
+          gtk_widget_show (button);
 
-      row = n / rows;
-      col = n % rows;
+          g_object_set_data_full (G_OBJECT (button), "viewport-info", vp_info,
+                                  (GDestroyNotify) g_free);
 
-      gtk_table_attach (GTK_TABLE (pager), button,
-                        row, row + 1, col, col + 1,
-                        GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
-                        0, 0);
+          g_snprintf (text, sizeof (text), "%d", n + 1);
+          label = gtk_label_new (text);
+          gtk_label_set_angle (GTK_LABEL (label),
+              pager->orientation == GTK_ORIENTATION_HORIZONTAL ? 0 : 270);
+          gtk_container_add (GTK_CONTAINER (button), label);
+          gtk_widget_show (label);
+
+          row = n / rows;
+          col = n % rows;
+
+          gtk_table_attach (GTK_TABLE (pager), button,
+                            row, row + 1, col, col + 1,
+                            GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                            0, 0);
+        }
+    }
+  else
+    {
+      for (li = workspaces, n = 0; li != NULL; li = li->next, n++)
+        {
+          workspace = WNCK_WORKSPACE (li->data);
+
+          button = xfce_panel_create_toggle_button ();
+          if (workspace == active_ws)
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+          g_signal_connect (G_OBJECT (button), "toggled",
+              G_CALLBACK (pager_buttons_workspace_button_toggled), workspace);
+          g_signal_connect (G_OBJECT (button), "button-press-event",
+              G_CALLBACK (pager_buttons_button_press_event), NULL);
+          xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (panel_plugin), button);
+          gtk_widget_show (button);
+
+          label = gtk_label_new (NULL);
+          g_signal_connect_object (G_OBJECT (workspace), "name-changed",
+              G_CALLBACK (pager_buttons_workspace_button_label), label, 0);
+          pager_buttons_workspace_button_label (workspace, label);
+          gtk_label_set_angle (GTK_LABEL (label),
+              pager->orientation == GTK_ORIENTATION_HORIZONTAL ? 0 : 270);
+          gtk_container_add (GTK_CONTAINER (button), label);
+          gtk_widget_show (label);
+
+          pager->buttons = g_slist_prepend (pager->buttons, button);
+
+          row = n / rows;
+          col = n % rows;
+
+          gtk_table_attach (GTK_TABLE (pager), button,
+                            row, row + 1, col, col + 1,
+                            GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                            0, 0);
+        }
     }
 
   pager->buttons = g_slist_reverse (pager->buttons);
@@ -383,7 +507,6 @@ pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
 {
   panel_return_if_fail (WNCK_IS_SCREEN (screen));
   panel_return_if_fail (WNCK_IS_WORKSPACE (destroyed_workspace));
-  panel_return_if_fail (WNCK_IS_WORKSPACE (destroyed_workspace));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
   panel_return_if_fail (pager->wnck_screen == screen);
 
@@ -392,6 +515,22 @@ pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
 
 
 
+static void
+pager_buttons_screen_viewports_changed (WnckScreen    *screen,
+                                        PagerButtons  *pager)
+{
+  panel_return_if_fail (WNCK_IS_SCREEN (screen));
+  panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
+  panel_return_if_fail (pager->wnck_screen == screen);
+
+  /* yes we are extremely lazy here, but this event is
+   * also emitted when the viewport setup changes... */
+  if (pager->buttons == NULL)
+    pager_buttons_queue_rebuild (pager);
+}
+
+
+
 static void
 pager_buttons_workspace_button_label (WnckWorkspace *workspace,
                                       GtkWidget     *label)
@@ -439,6 +578,27 @@ pager_buttons_workspace_button_toggled (GtkWidget     *button,
 
 
 
+static void
+pager_buttons_viewport_button_toggled (GtkWidget    *button,
+                                       PagerButtons *pager)
+{
+  gint *vp_info;
+
+  panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (button));
+  panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
+  panel_return_if_fail (WNCK_IS_SCREEN (pager->wnck_screen));
+
+  vp_info = g_object_get_data (G_OBJECT (button), "viewport-info");
+  if (G_UNLIKELY (vp_info == NULL))
+    return;
+
+  wnck_screen_move_viewport (pager->wnck_screen,
+                             vp_info[VIEWPORT_X],
+                             vp_info[VIEWPORT_Y]);
+}
+
+
+
 GtkWidget *
 pager_buttons_new (WnckScreen *screen)
 {
