@@ -132,7 +132,7 @@ pager_plugin_class_init (PagerPluginClass *klass)
                                    PROP_MINIATURE_VIEW,
                                    g_param_spec_boolean ("miniature-view",
                                                          NULL, NULL,
-                                                         FALSE,
+                                                         TRUE,
                                                          EXO_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
@@ -141,8 +141,6 @@ pager_plugin_class_init (PagerPluginClass *klass)
                                                       NULL, NULL,
                                                       1, 50, 1,
                                                       EXO_PARAM_READWRITE));
-
-
 }
 
 
@@ -152,7 +150,7 @@ pager_plugin_init (PagerPlugin *plugin)
 {
   plugin->wnck_screen = NULL;
   plugin->scrolling = TRUE;
-  plugin->miniature_view = FALSE;
+  plugin->miniature_view = TRUE;
   plugin->rows = 1;
   plugin->pager = NULL;
 }
@@ -215,7 +213,11 @@ pager_plugin_set_property (GObject      *object,
       if (plugin->pager != NULL)
         {
           if (plugin->miniature_view)
-            wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
+            {
+              if (!wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows))
+                g_message ("Failed to set the number of pager rows. You probably "
+                           "have more then 1 pager in your panel setup.");
+            }
           else
             pager_buttons_set_n_rows (XFCE_PAGER_BUTTONS (plugin->pager), plugin->rows);
         }
@@ -290,7 +292,9 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin)
     {
       plugin->pager = wnck_pager_new (plugin->wnck_screen);
       wnck_pager_set_display_mode (WNCK_PAGER (plugin->pager), WNCK_PAGER_DISPLAY_CONTENT);
-      wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
+      if (!wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows))
+        g_message ("Failed to set the number of pager rows. You probably "
+                   "have more then 1 pager in your panel setup.");
       wnck_pager_set_orientation (WNCK_PAGER (plugin->pager), orientation);
     }
   else
@@ -422,7 +426,7 @@ pager_plugin_configure_workspace_settings (GtkWidget *button)
       /* show an error dialog */
       toplevel = gtk_widget_get_toplevel (button);
       xfce_dialog_show_error (GTK_WINDOW (toplevel), error,
-          _("Unable to open the Xfce workspace settings"));
+          _("Unable to open the workspace settings"));
       g_error_free (error);
     }
 }
@@ -434,18 +438,31 @@ pager_plugin_configure_n_workspaces_changed (WnckScreen    *wnck_screen,
                                              WnckWorkspace *workspace,
                                              GtkBuilder    *builder)
 {
-  GObject *object;
-  gdouble  n_worspaces, value;
+  GObject       *object;
+  gdouble        upper, value;
+  WnckWorkspace *active_ws;
 
   panel_return_if_fail (WNCK_IS_SCREEN (wnck_screen));
   panel_return_if_fail (GTK_IS_BUILDER (builder));
 
   object = gtk_builder_get_object (builder, "rows");
 
-  n_worspaces = wnck_screen_get_workspace_count (wnck_screen);
-  value = MIN (gtk_adjustment_get_value (GTK_ADJUSTMENT (object)), n_worspaces);
+  upper = wnck_screen_get_workspace_count (wnck_screen);
+  if (upper == 1)
+    {
+      /* check if we ware in viewport mode */
+      active_ws = wnck_screen_get_active_workspace (wnck_screen);
+      if (wnck_workspace_is_virtual (active_ws))
+        {
+          /* number of rows * number of columns */
+          upper = (wnck_workspace_get_width (active_ws) / wnck_screen_get_width (wnck_screen))
+                  * (wnck_workspace_get_height (active_ws) / wnck_screen_get_height (wnck_screen));
+        }
+    }
 
-  g_object_set (G_OBJECT (object), "upper", n_worspaces, "value", value, NULL);
+  value = MIN (gtk_adjustment_get_value (GTK_ADJUSTMENT (object)), upper);
+
+  g_object_set (G_OBJECT (object), "upper", upper, "value", value, NULL);
 }
 
 
