@@ -448,6 +448,7 @@ systray_box_size_allocate (GtkWidget     *widget,
   gint             child_size;
   GtkAllocation    child_allocation;
   gint             swap;
+  gint             n_children;
 
   panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (widget));
   panel_return_if_fail (allocation != NULL);
@@ -455,6 +456,10 @@ systray_box_size_allocate (GtkWidget     *widget,
   /* set widget allocation */
   widget->allocation = *allocation;
 
+  n_children = g_slist_length (box->childeren);
+  if (n_children == 0)
+    return;
+
   /* get root coordinates */
   x = allocation->x + GTK_CONTAINER (widget)->border_width;
   y = allocation->y + GTK_CONTAINER (widget)->border_width;
@@ -464,9 +469,27 @@ systray_box_size_allocate (GtkWidget     *widget,
   height = allocation->height - 2 * GTK_CONTAINER (widget)->border_width;
 
   /* child size */
-  child_size = IS_HORIZONTAL (box) ? height : width;
-  child_size -= SPACING * (box->rows - 1);
-  child_size /= box->rows;
+  if (box->rows == 1)
+    {
+      child_size = IS_HORIZONTAL (box) ? width : height;
+      if (box->n_hidden_childeren > 0)
+         child_size -= BUTTON_SIZE + SPACING;
+      n = n_children - (box->show_hidden ? 0 : box->n_hidden_childeren);
+      child_size -= SPACING * MAX (n - 1, 0);
+      if (n > 1)
+        child_size /= n;
+
+      if (IS_HORIZONTAL (box))
+        y += MAX (height - child_size, 0) / 2;
+      else
+        x += MAX (width - child_size, 0) / 2;
+    }
+  else
+    {
+      child_size = IS_HORIZONTAL (box) ? height : width;
+      child_size -= SPACING * (box->rows - 1);
+      child_size /= box->rows;
+    }
 
   /* don't allocate zero width icon */
   if (child_size < 1)
@@ -476,8 +499,8 @@ systray_box_size_allocate (GtkWidget     *widget,
   if (box->n_hidden_childeren > 0)
     {
       /* initialize allocation */
-      child_allocation.x = x;
-      child_allocation.y = y;
+      child_allocation.x = allocation->x + GTK_CONTAINER (widget)->border_width;
+      child_allocation.y = allocation->y + GTK_CONTAINER (widget)->border_width;
 
       /* set the width and height */
       if (IS_HORIZONTAL (box))
