@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005-2007 Jasper Huijsmans <jasper@xfce.org>
- * Copyright (c) 2007-2009 Nick Schermer <nick@xfce.org>
+ * Copyright (c) 2007-2010 Nick Schermer <nick@xfce.org>
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -85,6 +85,8 @@ struct _SystrayPlugin
   /* systray manager */
   SystrayManager *manager;
 
+  guint           idle_startup;
+
   /* widgets */
   GtkWidget      *frame;
   GtkWidget      *box;
@@ -168,6 +170,7 @@ systray_plugin_init (SystrayPlugin *plugin)
 {
   plugin->manager = NULL;
   plugin->show_frame = TRUE;
+  plugin->idle_startup = 0;
 
   plugin->frame = gtk_frame_new (NULL);
   gtk_container_add (GTK_CONTAINER (plugin), plugin->frame);
@@ -244,59 +247,70 @@ systray_plugin_set_property (GObject      *object,
 
 
 
+static gboolean
+systray_plugin_screen_changed_idle (gpointer user_data)
+{
+  SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (user_data);
+  GdkScreen     *screen;
+  GError        *error = NULL;
+
+  /* create a new manager and register this screen */
+  plugin->manager = systray_manager_new ();
+  g_signal_connect (G_OBJECT (plugin->manager), "icon-added",
+      G_CALLBACK (systray_plugin_icon_added), plugin);
+  g_signal_connect (G_OBJECT (plugin->manager), "icon-removed",
+      G_CALLBACK (systray_plugin_icon_removed), plugin);
+  g_signal_connect (G_OBJECT (plugin->manager), "lost-selection",
+      G_CALLBACK (systray_plugin_lost_selection), plugin);
+
+  /* try to register the systray */
+  screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+  if (systray_manager_register (plugin->manager, screen, &error))
+    {
+      /* send the plugin orientation */
+      systray_plugin_orientation_changed (XFCE_PANEL_PLUGIN (plugin),
+         xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)));
+    }
+  else
+    {
+      xfce_dialog_show_error (NULL, error, _("Unable to start the notification area"));
+      g_error_free (error);
+    }
+
+  return FALSE;
+}
+
+
+
+static void
+systray_plugin_screen_changed_idle_destroyed (gpointer user_data)
+{
+  XFCE_SYSTRAY_PLUGIN (user_data)->idle_startup = 0;
+}
+
+
+
 static void
 systray_plugin_screen_changed (GtkWidget *widget,
                                GdkScreen *previous_screen)
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (widget);
-  GdkScreen     *screen;
-  GError        *error = NULL;
 
   if (G_UNLIKELY (plugin->manager != NULL))
     {
-      /* unregister from this screen */
+      /* unregister this screen screen */
       systray_manager_unregister (plugin->manager);
       g_object_unref (G_OBJECT (plugin->manager));
       plugin->manager = NULL;
     }
 
-  /* check if not another systray is running on this screen */
-  screen = gtk_widget_get_screen (widget);
-  if (G_LIKELY (!systray_manager_check_running (screen)))
-    {
-      /* create a new manager and register this screen */
-      plugin->manager = systray_manager_new ();
-
-      /* hookup signals */
-      g_signal_connect (G_OBJECT (plugin->manager), "icon-added",
-          G_CALLBACK (systray_plugin_icon_added), plugin);
-      g_signal_connect (G_OBJECT (plugin->manager), "icon-removed",
-          G_CALLBACK (systray_plugin_icon_removed), plugin);
-      g_signal_connect (G_OBJECT (plugin->manager), "lost-selection",
-          G_CALLBACK (systray_plugin_lost_selection), plugin);
-
-      if (systray_manager_register (plugin->manager, screen, &error))
-        {
-          /* send the plugin orientation */
-          systray_plugin_orientation_changed (XFCE_PANEL_PLUGIN (plugin),
-              xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)));
-        }
-      else
-        {
-          /* TODO handle error and leave the plugin */
-          g_message ("Failed to register the systray manager %s", error->message);
-          g_error_free (error);
-        }
-    }
-  else
-    {
-      /* TODO, error and leave the plugin */
-      g_message ("already a notification area running");
-    }
+  /* schedule a delayed startup */
+  if (plugin->idle_startup == 0)
+    plugin->idle_startup = g_idle_add_full (G_PRIORITY_LOW, systray_plugin_screen_changed_idle,
+                                            plugin, systray_plugin_screen_changed_idle_destroyed);
 }
 
 
-#define LAUNCHER_TYPE_PTR_ARRAY        (dbus_g_type_get_collection("GPtrArray", G_TYPE_VALUE))
 
 static void
 systray_plugin_construct (XfcePanelPlugin *panel_plugin)
@@ -338,6 +352,10 @@ systray_plugin_free_data (XfcePanelPlugin *panel_plugin)
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
 
+  /* stop pending idle startup */
+  if (plugin->idle_startup != 0)
+    g_source_remove (plugin->idle_startup);
+
   /* disconnect screen changed signal */
   g_signal_handlers_disconnect_by_func (G_OBJECT (plugin),
       systray_plugin_screen_changed, NULL);
@@ -549,9 +567,8 @@ systray_plugin_lost_selection (SystrayManager *manager,
 
   /* create fake error and show it */
   error.message = _("Most likely another widget took over the function "
-                    "of a notification area. This plugin will close.");
-  xfce_dialog_show_error (NULL, &error,
-                          _("The notification area lost selection"));
+                    "of a notification area. This area will be unused.");
+  xfce_dialog_show_error (NULL, &error, _("The notification area lost selection"));
 }
 
 
