@@ -132,6 +132,7 @@ tasklist_plugin_construct (XfcePanelPlugin *panel_plugin)
     { "show-wireframes", G_TYPE_BOOLEAN },
     { "show-handle", G_TYPE_BOOLEAN },
     { "sort-order", G_TYPE_UINT },
+    { "rotate-vertically", G_TYPE_BOOLEAN },
     { NULL }
   };
 
@@ -185,6 +186,7 @@ tasklist_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   GtkBuilder     *builder;
   GObject        *dialog;
   GObject        *object;
+  GtkTreeIter     iter;
 
   /* setup the dialog */
   PANEL_UTILS_LINK_4UI
@@ -210,6 +212,7 @@ tasklist_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   TASKLIST_DIALOG_BIND ("grouping", "active")
   TASKLIST_DIALOG_BIND ("include-all-workspaces", "active")
   TASKLIST_DIALOG_BIND ("flat-buttons", "active")
+  TASKLIST_DIALOG_BIND ("rotate-vertically", "active")
   TASKLIST_DIALOG_BIND_INV ("switch-workspace-on-unminimize", "active")
   TASKLIST_DIALOG_BIND ("show-only-minimized", "active")
   TASKLIST_DIALOG_BIND ("show-wireframes", "active")
@@ -222,6 +225,11 @@ tasklist_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   gtk_widget_hide (GTK_WIDGET (object));
 #endif
 
+  /* TODO: remove this if always group is supported */
+  object = gtk_builder_get_object (builder, "grouping-model");
+  if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (object), &iter, NULL, 2))
+    gtk_list_store_remove (GTK_LIST_STORE (object), &iter);
+
   gtk_widget_show (GTK_WIDGET (dialog));
 }
 
